#include "board.h"

/**
 * Create the board of the game
 * @param int width of the board
 * @param int height of the board
 * @return return the created board
 */
struct board create_board(int col, int line){
    struct board board;
    board.data = malloc(sizeof(struct board) * line); // Dynamically allocates memory for the board height
    for(int i = 0; i < line; i++){
        board.data[i] = malloc(sizeof(struct board)*col); // Dynamically allocates memory for the board width for each board line
    }
    // Filling the different members of the struct
    board.col = col;
    board.line = line;
    board.last_pos_x = -1;
    board.last_pos_y = -1;
    board.last_symbole = Vide;

    return board;
}


/**
 * Initilize the board with the enum "Empty"
 * @param struct the pointer of the game board
 * @return no return 
 */
void init_board(struct board *board){
    for(int i = 0; i < board->line;i++){
        for(int j = 0; j < board->col; j++){
            board->data[i][j] = Vide; // Each cases get enum "Empty"
        }
    }
}

/**
 * Print the game board
 * @param struct the pointer of the game board
 * @return no return
 */
void print_game(struct board *board){
    printf("┌");
    for(int i = 1; i < board->col; i++){
        printf("─┬");
    }
    printf("─┐\n");

    for(int i = 0; i < board->line;i++){
        printf("│");
        for(int j = 0; j < board->col; j++){
            if(board->data[i][j] == Vide){
                printf(" │");
            }
            if(board->data[i][j] == Croix){
                printf("X│");
            }
            if(board->data[i][j] == Cercle){
                printf("O│");
            }
        }
        if(i != board->line-1){
            printf("\n├");
            for(int i = 1; i < board->col; i++){
                printf("─┼");
            }
            printf("─┤\n");
        }
        else{
            printf("\n");
        }
    }
    printf("└");
    for(int i = 1; i < board->col; i++){
        printf("─┴");
    }
    printf("─┘\n");
    for(int i = 1; i <= board->col; i++){
        printf(" %d", i);
    }
    printf("\n");
}

/**
 * Look the next empty case of a column
 * @param struct the game board
 * @param int number of the colmun to look
 * @return return the line of the next empty case
 */
int current_line(struct board board, int no_col){
    int no_line = board.line - 1;
    while (no_line >= 0 && board.data[no_line][no_col] != Vide) { // Decrement no_line while the next case is not empty and no_line is bigger then 0
        no_line--;
    }
    return no_line;
}

/**
 * Look if the board is full
 * @param struct the game board
 * @return return if the board is full or not
 */
bool is_full_board(struct board board){
    for(int i = 0; i < board.line; i++){
        for(int j = 0; j < board.col; j++){
            if(board.data[i][j] == Vide){ // if one of case is empty, we return false
                return false;
            }
        }
    }
    return true;
}

/**
 * Free the game board when the game is over
 * @param struct the pointer of the game board
 */
void free_board(struct board *board){
    for(int i = 0; i < board->line;i++){
        free(board->data[i]); // free the width of lines
    }
    free(board->data); // free lines
}

