#include "smartAI.h"

/**
 * Simulation for the piece placement AI to see if it can win on the next round 
 * @param struct the pointer of the game board
 * @return return 0 if AI can win on the next round, 0 if it can't win 
 */
int simulate_for_win(struct board *board){
    int no_line = -1;
    for(int i = 0; i < board->col; i++){
        no_line = current_line(*board, i);
        if(board->data[no_line][i] == Vide){
            board->data[no_line][i] = Cercle; // Simulation to put a AI piece in the game
            board->last_symbole = Cercle;
            board->last_pos_x = no_line;
            board->last_pos_y = i;
            if(winner(*board)){ // if AI can win, we leave the simulated piece
                return 0;
            }
            board->data[no_line][i] = Vide; // Else, we remove the piece
            board->last_symbole = Vide;
            board->last_pos_x = -1;
            board->last_pos_y = -1;
        }
    }

    return -1;
}

/**
 * Simulation for the piece placement AI to block player to win
 * @param the pointer of the game board
 * @return return 0 if the AI ​​was able to block the player, 0 if it can't block the player
 */
int simulate_for_block_player(struct board *board){
    int no_line = -1;
    for(int i = 0; i < board->col; i++){
        no_line = current_line(*board, i);
        if(board->data[no_line][i] == Vide){
            board->data[no_line][i] = Croix; // Simulation to put a player piece in the game
            board->last_symbole = Croix;
            board->last_pos_x = no_line;
            board->last_pos_y = i;

            if(winner(*board)){ // If the AI can block, we put a AI piece in the same place
                board->data[no_line][i] = Cercle;
                board->last_symbole = Cercle;
                board->last_pos_x = no_line;
                board->last_pos_y = i;
                return 0;
            }
            board->data[no_line][i] = Vide; // Else, we remove the piece
            board->last_symbole = Vide;
            board->last_pos_x = -1;
            board->last_pos_y = -1;
        }
    }
    return -1;
}

/**
 * Actions for play with the smart AI
 */
void play_with_smartAI(struct board board){
    int no_col = 0;
    int no_line = 0;
    int no_round = 1;
    while(!is_full_board(board)){
        if(no_round % 2 != 0){
            printf("Column number? (starts at 1):");
            scanf("%d", &no_col);
            printf("\n");
            no_line = current_line(board,no_col - 1);
            board.data[no_line][no_col - 1] = Croix;
            board.last_symbole = Croix;
        }
        else{
            if(simulate_for_win(&board) != -1){ // If the AI can win, we stop the game and AI is the winner
                print_game(&board);
                printf("Computer won!\n");
                break;
            }
            
            if(simulate_for_block_player(&board) != -1){ // If the AI block the player, we continue with the next round
                print_game(&board);
                no_round++;
                continue;
            }
            no_col = rand() % 7; // Else, AI place his piece in the random column
            no_line = current_line(board,no_col);
            board.data[no_line][no_col] = Cercle;
            board.last_symbole = Cercle;
        }
        board.last_pos_x = no_line;
        board.last_pos_y = no_col - 1;
        print_game(&board);
        if(winner(board)){
            if(board.last_symbole == Croix){
                printf("Player one won!\n");
            }
            else if(board.last_symbole == Cercle){
                printf("Computer won!\n");
            }
            break;
        }
        no_round++;
    }
    if(is_full_board(board)){
        printf("It is a draw.\n");
    }
}